/*
  Copyright (C) 2006-2009 Alessandro Bugatti (alessandro.bugatti@istruzione.it)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/*! \file
 *  \brief Esempio di utilizo di quadriche
 *  \author Alessandro Bugatti
 *  \version 0.3
 *  \date  Creazione 04/02/2009
 *  \date  Ultima modifica 10/02/2014
 *  \attention Questo programma per funzionare sotto Windows ha bisogno
 *  della presenza della libreria SDL.dll che pu essere installata
 *  nella cartella di sistema o nela cartella dell'eseguibile.
 *  Nelle distribuzioni Linux la libreria SDL di solito  installata
 *  di default.
 */


#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <ctime>
#include <cmath>
#include <sstream>
#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>
using namespace std;

#define SCREEN_WIDTH  640   //Larghezza dello schermo
#define SCREEN_HEIGHT 480   //Altezza dello schermo
#define SCREEN_BPP     16   //Profondit di colore




// Luce ambiente
GLfloat LightAmbient[]  = { 0.2f, 0.2f, 0.3f, 1.0f };
// Luce diffusa
GLfloat LightDiffuse[]  = { 1.0f, 1.0f, 1.0f, 1.0f };
//Posizione della luce
GLfloat Light1Position[] = { 10.0f, 1.0f, -10.0f, 1.0f };
GLfloat Light2Position[] = { 1.0f, -10.0f, -10.0f, 1.0f };
bool toggle_light_2 = true;

GLenum actual_style = GLU_FILL;

GLUquadricObj *sfera, *cilindro;
/* Funzione per inizializzare OpenGL */
bool initGL()
{
    /*Imposto il viewport
    glViewport(x, y, larghezza, altezza)*/
    glViewport(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
    /*Imposto il colore di sfondo, quello utilizzato per "ripitturare"
    glClearColor(rosso, verde , blu, canale_alfa)*/
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    /*Imposto il depth test*/
    glClearDepth(1.0);
    glDepthFunc(GL_LESS);
    glEnable(GL_DEPTH_TEST);
    /* Inizializzo la prospettiva */
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );
    GLfloat ratio = ( GLfloat )SCREEN_WIDTH / SCREEN_HEIGHT;
    gluPerspective( 45.0f, ratio, 8.1f, 50.0f );
    /* Inizializzo la camera */
    //Abilita l'illuminazione
    glEnable(GL_LIGHTING);
    //Definisco le propriet dela luce ambiente
    glLightfv( GL_LIGHT1, GL_AMBIENT, LightAmbient );
    //Definisco le propriet dela luce diffusa
    glLightfv( GL_LIGHT1, GL_DIFFUSE, LightDiffuse );
    //Definisco la posizione della luce
    glLightfv( GL_LIGHT1, GL_POSITION, Light1Position );
    //Definisco le propriet dela luce ambiente
    glLightfv( GL_LIGHT2, GL_AMBIENT, LightAmbient );
    //Definisco le propriet dela luce diffusa
    glLightfv( GL_LIGHT2, GL_DIFFUSE, LightDiffuse );
    //Definisco la posizione della luce
    glLightfv( GL_LIGHT2, GL_POSITION, Light2Position );
    /* Abilito la prima luce */
    glEnable( GL_LIGHT1 );
    glDisable( GL_LIGHT2 );
    glMatrixMode( GL_PROJECTION );
    //glLoadIdentity();
    gluLookAt( 0, 5, -10, 0  , 0, -30, 0, 1, 0);
    return true;
}

int drawGLScene(  )
{
    /* Servono se si vogliono calcolare i fps (frame per second) */
    static GLint T0     = 0;
    static GLint Frames = 0;
    static GLint T1     = 0;
    /* Pulisce i buffer */
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity( );

    //Disegno della sfera
        glTranslatef(-5,0,-30);
        gluQuadricDrawStyle(sfera, GLU_POINT);
        GLfloat ambient[]={1,0,0,1};
        glMaterialfv(GL_FRONT,GL_AMBIENT,ambient);
        GLfloat diffuse[]={1,0,0,1};
        glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse);
        GLfloat specular[]={1,0,0,1};
        glMaterialfv(GL_FRONT,GL_SPECULAR,specular);
        glMaterialf(GL_FRONT, GL_SHININESS,25);
        gluSphere(sfera,
                   5,
                   40,
                   40);
    //Disegno del cilindro
        glLoadIdentity();
        glTranslatef(5,5,-30);
        glRotatef(90,1,0,0);
        gluQuadricDrawStyle(cilindro, GLU_LINE);
        GLfloat ambient1[]={0,1,0,1};
        glMaterialfv(GL_FRONT,GL_AMBIENT,ambient1);
        GLfloat diffuse1[]={0,1,0,1};
        glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse1);
        GLfloat specular1[]={0,1,0,1};
        glMaterialfv(GL_FRONT,GL_SPECULAR,specular1);
        glMaterialf(GL_FRONT, GL_SHININESS,25);
        gluCylinder(cilindro,
                   0,
                   5,
                   10,
                   4,
                   4);
    /* Disegna a schermo scambiando i buffer */
    SDL_GL_SwapBuffers( );

    /* Incrementa il numero di frame */
    Frames++;
    {
	GLint t = SDL_GetTicks();
	//Fa il calcolo ogni 5 secondi
	if (t - T0 >= 5000) {
	    GLfloat seconds = (t - T0) / 1000.0;
	    GLfloat fps = Frames / seconds;
	    cout << "Frames al secondo = " << fps << " FPS\n";

	    ostringstream o;
	    o << "Quadriche - FPS : " << fps;
	    SDL_WM_SetCaption(o.str().c_str(),"");
	    T0 = t;
	    Frames = 0;
	}

	}

    return true ;
}


int main( int argc, char **argv )
{
    /* Variabile per terminare il main loop */
    int done = false;
    /* Viariabile per leggere gli eventi */
    SDL_Event event;

    /* Inizializzo SDL */
    if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
    {
        cerr << "Inizializzazione SDL fallita: " <<
        SDL_GetError( ) << endl;
        exit(1);
    }
    //Registro la funzione da chiamare in caso di terminazione
    //SDL_Quit "rimette a posto" tutto
    atexit (SDL_Quit);
    //Setto il titolo della finestra
    SDL_WM_SetCaption("Quadriche","");
    /*Abilito il double buffering (che comunque  abilitato di default)*/
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
    /*Setto la modalit grafica
    SDL_SetVideoMode(larghezza, altezza, profondit_di_colore, attributi)*/
    SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 0, SDL_OPENGL | SDL_HWSURFACE);
    /* Inizializzo OpenGL */
    initGL();
    glMatrixMode(GL_MODELVIEW);

    sfera = gluNewQuadric();
    cilindro = gluNewQuadric();
    cout << glGetString(GL_VERSION) << endl;
    /* main loop: continuo a ridisegnare e quando arriva qualche
       evento lo processo */
    while (!done)
    {
        //Legge la coda degli eventi e riempie la struttura event
        while ( SDL_PollEvent( &event ) )
        {
            switch ( event.type )
            {
            case SDL_KEYDOWN:
                /* gestisco la pressione di un tasto */
                switch ( event.key.keysym.sym )
                {
                case SDLK_ESCAPE: //premuto il tasto ESC
                    exit(0);
                    break;
                case SDLK_a: //premuto il tasto a abilita l'illuminazione
            	    glEnable(GL_LIGHTING);
            	    break;
        	    case SDLK_s: //premuto il tasto s disabilita l'illuminazione
            	    glDisable(GL_LIGHTING);
            	    break;
                case SDLK_2: //premuto il tasto 2 abilita/disabilita la luce 2
            	    (toggle_light_2=!toggle_light_2)?glDisable(GL_LIGHT2):glEnable(GL_LIGHT2);
            	    break;
        	    default:
                    break;
                }
                break;
            case SDL_QUIT:
                done = true; //Gestisco la richiesta di uscita
                break;
            default:
                break;
            }
        }

        /* Disegno la scena */
        drawGLScene( );
    }
    return 0 ;
}
